//***************** (c) 2010 BTC Korporacja ***********************************
//                     http://www kamami.com
//
//    THE SOFTWARE INCLUDED IN THIS FILE IS FOR GUIDANCE ONLY.
//    BTC KORPORACJA SHALL NOT BE HELD LIABLE FOR ANY DIRECT, INDIRECT
//    OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
//    FROM USE OF THIS SOFTWARE.
//
//******************************************************************************
#include "HD44780.h"
#include "stm32f10x_lib.h"
#include "board.h"

GPIO_InitTypeDef GPIO_InitStructure;

//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void LCD_WriteNibble(u8 nibbleToWrite)
{
	GPIO_WriteBit(LCD_GPIO, LCD_EN, Bit_SET);
	GPIO_WriteBit(LCD_GPIO, LCD_D4, (nibbleToWrite & 0x01));
	GPIO_WriteBit(LCD_GPIO, LCD_D5, (nibbleToWrite & 0x02));
	GPIO_WriteBit(LCD_GPIO, LCD_D6, (nibbleToWrite & 0x04));
	GPIO_WriteBit(LCD_GPIO, LCD_D7, (nibbleToWrite & 0x08));
	GPIO_WriteBit(LCD_GPIO, LCD_EN, Bit_RESET);
}
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
u8 LCD_ReadNibble(void)
{
	u8 tmp = 0;
	GPIO_WriteBit(LCD_GPIO, LCD_EN, Bit_SET);
	tmp |= (GPIO_ReadInputDataBit(LCD_GPIO, LCD_D4) << 0);
	tmp |= (GPIO_ReadInputDataBit(LCD_GPIO, LCD_D5) << 1);
	tmp |= (GPIO_ReadInputDataBit(LCD_GPIO, LCD_D6) << 2);
	tmp |= (GPIO_ReadInputDataBit(LCD_GPIO, LCD_D7) << 3);
	GPIO_WriteBit(LCD_GPIO, LCD_EN, Bit_RESET);
	return tmp;
}
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
u8 LCD_ReadStatus(void)
{
	u8 status = 0;
	
	GPIO_InitStructure.GPIO_Pin   =  LCD_D4 | LCD_D5 | LCD_D6 | LCD_D7;
	GPIO_InitStructure.GPIO_Mode  =  GPIO_Mode_IPU;
	GPIO_Init(LCD_GPIO, &GPIO_InitStructure);
	
	GPIO_WriteBit(LCD_GPIO, LCD_RW, Bit_SET);
	GPIO_WriteBit(LCD_GPIO, LCD_RS, Bit_RESET);
	
	status |= (LCD_ReadNibble() << 4);
	status |= LCD_ReadNibble();
	
	GPIO_InitStructure.GPIO_Pin   =  LCD_D4 | LCD_D5 | LCD_D6 | LCD_D7;
	GPIO_InitStructure.GPIO_Mode  =  GPIO_Mode_Out_PP;
	GPIO_Init(LCD_GPIO, &GPIO_InitStructure);
	
	return status;
}
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void LCD_WriteData(u8 dataToWrite)
{
	GPIO_WriteBit(LCD_GPIO, LCD_RW, Bit_RESET);
	GPIO_WriteBit(LCD_GPIO, LCD_RS, Bit_SET);
	
	LCD_WriteNibble(dataToWrite >> 4);
	LCD_WriteNibble(dataToWrite & 0x0F);
	
	while(LCD_ReadStatus() & 0x80);
}
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void LCD_WriteCommand(u8 commandToWrite)
{
	GPIO_WriteBit(LCD_GPIO, LCD_RW | LCD_RS, Bit_RESET);
	LCD_WriteNibble(commandToWrite >> 4);
	LCD_WriteNibble(commandToWrite & 0x0F);
	
	while(LCD_ReadStatus() & 0x80);
}
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void LCD_WriteText(char * text)
{
	while(*text)
	  LCD_WriteData(*text++);
}
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void LCD_GoTo(unsigned char x, unsigned char y)
{
	LCD_WriteCommand(HD44780_DDRAM_SET | (x + (0x40 * y)));
}
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void LCD_WriteTextXY(char * text, u8 x, u8 y)
{
	LCD_GoTo(x,y);
	while(*text)
	  LCD_WriteData(*text++);
}
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void LCD_WriteBinary(u32 var, u8 bitCount)
{
	s8 i;
	
	for(i = (bitCount - 1); i >= 0; i--)
	{
		LCD_WriteData((var & (1 << i))?'1':'0');
	}
}
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void LCD_ShiftLeft(void)
{
	LCD_WriteCommand(HD44780_DISPLAY_CURSOR_SHIFT | HD44780_SHIFT_LEFT | HD44780_SHIFT_DISPLAY);
}
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void LCD_ShiftRight(void)
{
	LCD_WriteCommand(HD44780_DISPLAY_CURSOR_SHIFT | HD44780_SHIFT_RIGHT | HD44780_SHIFT_DISPLAY);
}
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
void LCD_Initialize(void)
{
	vu8 i = 0;
	vu32 delayCnt = 0;
	
	GPIO_InitStructure.GPIO_Pin   =  LCD_D4|LCD_D5|LCD_D6|LCD_D7|LCD_RS|LCD_RW|LCD_EN;
	GPIO_InitStructure.GPIO_Speed =  GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode  =  GPIO_Mode_Out_PP;
	
	GPIO_Init(LCD_GPIO, &GPIO_InitStructure);
	
	GPIO_ResetBits(LCD_GPIO, LCD_RS | LCD_EN | LCD_RW);
	
	for(delayCnt = 0; delayCnt < 300000; delayCnt++);
	
	for(i = 0; i < 3; i++)              
	{
		LCD_WriteNibble(0x03);            
		for(delayCnt = 0;delayCnt < 30000; delayCnt++);
	}
	
	LCD_WriteNibble(0x02);             
	
	for(delayCnt = 0;delayCnt < 6000; delayCnt++);
	   
	LCD_WriteCommand(HD44780_FUNCTION_SET | 
	                 HD44780_FONT5x7 | 
	                 HD44780_TWO_LINE | 
	                 HD44780_4_BIT);
	
	LCD_WriteCommand(HD44780_DISPLAY_ONOFF | 
	                 HD44780_DISPLAY_OFF); 
	
	LCD_WriteCommand(HD44780_CLEAR); 
	
	LCD_WriteCommand(HD44780_ENTRY_MODE | 
	                 HD44780_EM_SHIFT_CURSOR | 
	                 HD44780_EM_INCREMENT);
	
	LCD_WriteCommand(HD44780_DISPLAY_ONOFF | 
	                 HD44780_DISPLAY_ON |
	                 HD44780_CURSOR_OFF | 
	                 HD44780_CURSOR_NOBLINK);
}
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
